@@ -9,6 +9,8 @@
 struct mail_user;
 struct mail_namespace;
 struct mail_storage;
+struct message_address;
+struct smtp_address;
 
 struct mail_storage_settings {
 	const char *mail_location;
@@ -58,6 +60,9 @@ struct mail_storage_settings {
 	const char *lock_method;
 	const char *pop3_uidl_format;
 
+	const char *hostname;
+	const char *recipient_delimiter;
+
 	const char *ssl_client_ca_dir;
 	const char *ssl_client_ca_file;
 	const char *ssl_crypto_device;
@@ -69,7 +74,6 @@ struct mail_storage_settings {
 	const char *const *parsed_mail_attachment_content_type_filter;
 	bool parsed_mail_attachment_exclude_inlined;
 	bool parsed_mail_attachment_detection_add_flags_on_save;
-	bool parsed_mail_attachment_detection_add_flags_on_fetch;
 };
 
 struct mail_namespace_settings {
@@ -128,8 +132,16 @@ struct mail_user_settings {
 
 	const char *mail_log_prefix;
 
+	const char *hostname;
+	const char *postmaster_address;
+
 	ARRAY(struct mail_namespace_settings *) namespaces;
 	ARRAY(const char *) plugin_envs;
+
+	/* May be NULL - use mail_storage_get_postmaster_address() instead of
+	   directly accessing this. */
+	const struct message_address *_parsed_postmaster_address;
+	const struct smtp_address *_parsed_postmaster_address_smtp;
 };
 
 extern const struct setting_parser_info mail_user_setting_parser_info;
@@ -148,10 +160,15 @@ mail_user_set_get_storage_set(struct mail_user *user);
 /* Get storage-specific settings, which may be namespace-specific. */
 const void *mail_namespace_get_driver_settings(struct mail_namespace *ns,
 					       struct mail_storage *storage);
-/* FIXME: Obsolete - remove in v2.3 */
-const void *mail_storage_get_driver_settings(struct mail_storage *storage);
 
 const struct dynamic_settings_parser *
 mail_storage_get_dynamic_parsers(pool_t pool);
 
+bool mail_user_set_get_postmaster_address(const struct mail_user_settings *set,
+					  const struct message_address **address_r,
+					  const char **error_r);
+bool mail_user_set_get_postmaster_smtp(const struct mail_user_settings *set,
+				       const struct smtp_address **address_r,
+				       const char **error_r);
+
 #endif
